<?php

namespace Drupal\moderation_state_buttons_widget\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\content_moderation\Plugin\Field\FieldWidget\ModerationStateWidget;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Plugin implementation of the 'moderation_state_buttons_widget' widget.
 *
 * @FieldWidget(
 *   id = "moderation_state_buttons",
 *   label = @Translation("Moderation state buttons"),
 *   field_types = {
 *     "string"
 *   }
 * )
 */
class ModerationStateButtonsWidget extends ModerationStateWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $items->getEntity();

    $workflow = $this->moderationInformation->getWorkflowForEntity($entity);
    $default = $items->get($delta)->value ? $workflow->getTypePlugin()->getState($items->get($delta)->value) : $workflow->getTypePlugin()->getInitialState($entity);
    $wrapperId = "moderation_state_columns--{$items->getName()}--$delta";
    $element['#prefix'] = "<div id=\"$wrapperId\">";
    $element['#suffix'] = "</div>";

    /** @var \Drupal\workflows\Transition[] $transitions */
    $transitions = $this->validator->getValidTransitions($entity, $this->currentUser);
    $transitionsMap = [];

    $default_value = NULL;
    foreach ($transitions as $transition) {
      $transition_to_state = $transition->to();
      $id = $transition_to_state->id();
      $label = $transition_to_state->label();
      $transitionsMap[$label] = $id;

      $element['buttons'][$id] = [
        '#type' => 'button',
        '#value' => $label,
        '#ajax' => [
          'callback' => [$this, 'moderationStateChanged'],
          'wrapper' => $wrapperId,
        ],
        '#limit_validation_errors' => [],
      ];
    }

    $triggeringElement = $form_state->getTriggeringElement();
    if ($triggeringElement && array_key_exists($triggeringElement['#value'], $transitionsMap)) {
      $selectedStateId = $transitionsMap[$triggeringElement['#value']];
    } else {
      $selectedStateId = $default->id();
    }

    $element['buttons'][$selectedStateId]['#attributes']['disabled'] = TRUE;

    $element += [
      $this->column => [
        '#type' => 'hidden',
        '#value' => $selectedStateId,
      ],
    ];

    $element['#attached']['library'][] = 'moderation_state_buttons_widget/widget';

    return $element;
  }

  /**
   * Returns the part of the form that needs to be replaced with AJAX.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return array
   */
  public function moderationStateChanged(array &$form, FormStateInterface $form_state, Request $request) {
    $buttonParents = $form_state->getTriggeringElement()['#array_parents'];
    return NestedArray::getValue($form, array_slice($buttonParents, 0, -2));
  }

  /**
   * {@inheritdoc}
   */
  public static function validateElement(array $element, FormStateInterface $form_state) {
    // We don't want to inherit this behavior from the parent plugin.
  }

}
